@@ -1,5 +1,6 @@
 /* { dg-do compile } */ 
-/* { dg-options "-O2 -fdump-tree-pre-stats" } */
+/* { dg-options "-O2 -fdump-tree-pre-stats -fdump-tree-fre1" } */
+#include <stddef.h>
 
 union tree_node;
 typedef union tree_node *tree;
@@ -63,7 +64,7 @@ main (void)
   for (i = 0; i < 2; i++)
     {
       unexpanded_var_list = malloc (sizeof (struct tree_list));
-      unexpanded_var_list->list.value = (tree) (long unsigned) (i & 1);
+      unexpanded_var_list->list.value = (tree) (ptrdiff_t) (i & 1);
       unexpanded_var_list->common.chain = last;
       last = unexpanded_var_list;
     }
@@ -71,6 +72,9 @@ main (void)
   remove_useless_vars (&unexpanded_var_list, 0);
   return 0;
 }
-/* { dg-final { scan-tree-dump-times "Eliminated: 2" 1 "pre" } } */
-/* { dg-final { cleanup-tree-dump "pre" } } */
 
+/* { dg-final { scan-tree-dump-not "= unexpanded_var_list;" "fre1" } } */
+/* { dg-final { scan-tree-dump-times "Eliminated: 1" 1 "pre" } } */
+/* { dg-final { scan-tree-dump-times "Insertions: 2" 1 "pre" } } */
+/* { dg-final { cleanup-tree-dump "fre1" } } */
+/* { dg-final { cleanup-tree-dump "pre" } } */
