@@ -6,7 +6,7 @@ program main
   if (.not. associated (x)) call abort
   if (size (x) .ne. 10) call abort
 contains
-  function test
+  function test()
     real, dimension (:), pointer :: test
     if (associated (x)) call abort
     allocate (test (10))
