@@ -1,15 +1,14 @@
 /* Ensure that overload resolution does not produce warnings as
    side-effects.  */
 /* { dg-do run } */
-/* { dg-additional-sources "../objc-obj-c++-shared/Object1.mm" } */
 /* { dg-xfail-run-if "Needs OBJC2 ABI" { *-*-darwin* && { lp64 && { ! objc2 } } } { "-fnext-runtime" } { "" } } */
 
-#include "../objc-obj-c++-shared/Object1.h"
+#include "../objc-obj-c++-shared/TestsuiteObject.m"
 #include <stdlib.h>
 
 #define CHECK_IF(E) if(!(E)) abort ()
 
-@interface MyCursor: Object
+@interface MyCursor: TestsuiteObject
 + (MyCursor *)crosshairCursor;
 @end
 
