@@ -138,6 +138,22 @@ SigTab runtime_sigtab[] = {
 #undef P
 #undef D
 
+
+static int8 badsignal[] = "runtime: signal received on thread not created by Go.\n";
+
+static void
+runtime_badsignal(int32 sig)
+{
+	// Avoid -D_FORTIFY_SOURCE problems.
+	int rv __attribute__((unused));
+
+	if (sig == SIGPROF) {
+		return;  // Ignore SIGPROFs intended for a non-Go thread.
+	}
+	rv = runtime_write(2, badsignal, sizeof badsignal - 1);
+	runtime_exit(1);
+}
+
 /* Handle a signal, for cases where we don't panic.  We can split the
    stack here.  */
 
@@ -146,11 +162,16 @@ sig_handler (int sig)
 {
   int i;
 
+  if (runtime_m () == NULL)
+    {
+      runtime_badsignal (sig);
+      return;
+    }
+
 #ifdef SIGPROF
   if (sig == SIGPROF)
     {
-      /* FIXME.  */
-      runtime_sigprof (0, 0, nil, nil);
+      runtime_sigprof ();
       return;
     }
 #endif
@@ -436,7 +457,7 @@ runtime_setsig (int32 i, bool def __attribute__ ((unused)), bool restart)
 
 /* Used by the os package to raise SIGPIPE.  */
 
-void os_sigpipe (void) __asm__ ("os.sigpipe");
+void os_sigpipe (void) __asm__ (GOSYM_PREFIX "os.sigpipe");
 
 void
 os_sigpipe (void)
