@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2006 Free Software Foundation
+/* Copyright (C) 2003, 2006, 2012 Free Software Foundation
 
    This file is part of libgcj.
 
@@ -10,11 +10,6 @@ details.  */
 #include <platform.h>
 #include <string.h>
 
-#if HAVE_BSTRING_H
-// Needed for bzero, implicitly used by FD_ZERO on IRIX 5.2
-#include <bstring.h>
-#endif
-
 #include <gnu/java/net/PlainDatagramSocketImpl.h>
 #include <java/io/IOException.h>
 #include <java/net/BindException.h>
@@ -545,8 +540,7 @@ gnu::java::net::PlainDatagramSocketImpl::setOption (jint optID,
       len = sizeof (struct in_addr);
       ptr = (const char *) &u.addr;
     }
-// Tru64 UNIX V5.0 has struct sockaddr_in6, but no IPV6_MULTICAST_IF
-#if defined (HAVE_INET6) && defined (IPV6_MULTICAST_IF)
+#ifdef HAVE_INET6
   else if (len == 16)
     {
       level = IPPROTO_IPV6;
@@ -643,14 +637,14 @@ gnu::java::net::PlainDatagramSocketImpl::getOption (jint optID)
         goto error;
       if (u.address.sin_family == AF_INET)
         {
-    laddr = JvNewByteArray (4);
-    memcpy (elements (laddr), &u.address.sin_addr, 4);
+	  laddr = JvNewByteArray (4);
+	  memcpy (elements (laddr), &u.address.sin_addr, 4);
         }
 #ifdef HAVE_INET6
-            else if (u.address.sin_family == AF_INET6)
+      else if (u.address.sin_family == AF_INET6)
         {
-    laddr = JvNewByteArray (16);
-    memcpy (elements (laddr), &u.address6.sin6_addr, 16);
+	  laddr = JvNewByteArray (16);
+	  memcpy (elements (laddr), &u.address6.sin6_addr, 16);
         }
 #endif
       else
