@@ -1,7 +1,6 @@
 // Multiset implementation -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-// Free Software Foundation, Inc.
+// Copyright (C) 2001-2013 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -49,18 +48,22 @@
  * purpose.  It is provided "as is" without express or implied warranty.
  */
 
-/** @file stl_multiset.h
+/** @file bits/stl_multiset.h
  *  This is an internal header file, included by other library headers.
- *  You should not attempt to use it directly.
+ *  Do not attempt to use it directly. @headername{set}
  */
 
 #ifndef _STL_MULTISET_H
 #define _STL_MULTISET_H 1
 
 #include <bits/concept_check.h>
+#if __cplusplus >= 201103L
 #include <initializer_list>
+#endif
 
-_GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
+namespace std _GLIBCXX_VISIBILITY(default)
+{
+_GLIBCXX_BEGIN_NAMESPACE_CONTAINER
 
   /**
    *  @brief A standard container made up of elements, which can be retrieved
@@ -68,6 +71,11 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
    *
    *  @ingroup associative_containers
    *
+   *
+   *  @tparam _Key  Type of key objects.
+   *  @tparam _Compare  Comparison function object type, defaults to less<_Key>.
+   *  @tparam _Alloc  Allocator type, defaults to allocator<_Key>.
+   *
    *  Meets the requirements of a <a href="tables.html#65">container</a>, a
    *  <a href="tables.html#66">reversible container</a>, and an
    *  <a href="tables.html#69">associative container</a> (using equivalent
@@ -118,7 +126,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       typedef typename _Rep_type::const_iterator            iterator;
       typedef typename _Rep_type::const_iterator            const_iterator;
       typedef typename _Rep_type::const_reverse_iterator    reverse_iterator;
-      typedef typename _Rep_type::const_reverse_iterator    const_reverse_iterator;
+      typedef typename _Rep_type::const_reverse_iterator const_reverse_iterator;
       typedef typename _Rep_type::size_type                 size_type;
       typedef typename _Rep_type::difference_type           difference_type;
 
@@ -131,22 +139,22 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
 
       /**
        *  @brief  Creates a %multiset with no elements.
-       *  @param  comp  Comparator to use.
-       *  @param  a  An allocator object.
+       *  @param  __comp  Comparator to use.
+       *  @param  __a  An allocator object.
        */
       explicit
       multiset(const _Compare& __comp,
 	       const allocator_type& __a = allocator_type())
-      : _M_t(__comp, __a) { }
+      : _M_t(__comp, _Key_alloc_type(__a)) { }
 
       /**
        *  @brief  Builds a %multiset from a range.
-       *  @param  first  An input iterator.
-       *  @param  last  An input iterator.
+       *  @param  __first  An input iterator.
+       *  @param  __last  An input iterator.
        *
        *  Create a %multiset consisting of copies of the elements from
        *  [first,last).  This is linear in N if the range is already sorted,
-       *  and NlogN otherwise (where N is distance(first,last)).
+       *  and NlogN otherwise (where N is distance(__first,__last)).
        */
       template<typename _InputIterator>
         multiset(_InputIterator __first, _InputIterator __last)
@@ -155,66 +163,67 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
 
       /**
        *  @brief  Builds a %multiset from a range.
-       *  @param  first  An input iterator.
-       *  @param  last  An input iterator.
-       *  @param  comp  A comparison functor.
-       *  @param  a  An allocator object.
+       *  @param  __first  An input iterator.
+       *  @param  __last  An input iterator.
+       *  @param  __comp  A comparison functor.
+       *  @param  __a  An allocator object.
        *
        *  Create a %multiset consisting of copies of the elements from
-       *  [first,last).  This is linear in N if the range is already sorted,
-       *  and NlogN otherwise (where N is distance(first,last)).
+       *  [__first,__last).  This is linear in N if the range is already sorted,
+       *  and NlogN otherwise (where N is distance(__first,__last)).
        */
       template<typename _InputIterator>
         multiset(_InputIterator __first, _InputIterator __last,
 		 const _Compare& __comp,
 		 const allocator_type& __a = allocator_type())
-	: _M_t(__comp, __a)
+	: _M_t(__comp, _Key_alloc_type(__a))
         { _M_t._M_insert_equal(__first, __last); }
 
       /**
        *  @brief  %Multiset copy constructor.
-       *  @param  x  A %multiset of identical element and allocator types.
+       *  @param  __x  A %multiset of identical element and allocator types.
        *
        *  The newly-created %multiset uses a copy of the allocation object used
-       *  by @a x.
+       *  by @a __x.
        */
       multiset(const multiset& __x)
       : _M_t(__x._M_t) { }
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
+#if __cplusplus >= 201103L
      /**
        *  @brief  %Multiset move constructor.
-       *  @param  x  A %multiset of identical element and allocator types.
+       *  @param  __x  A %multiset of identical element and allocator types.
        *
-       *  The newly-created %multiset contains the exact contents of @a x.
-       *  The contents of @a x are a valid, but unspecified %multiset.
+       *  The newly-created %multiset contains the exact contents of @a __x.
+       *  The contents of @a __x are a valid, but unspecified %multiset.
        */
       multiset(multiset&& __x)
-      : _M_t(std::forward<_Rep_type>(__x._M_t)) { }
+      noexcept(is_nothrow_copy_constructible<_Compare>::value)
+      : _M_t(std::move(__x._M_t)) { }
 
       /**
        *  @brief  Builds a %multiset from an initializer_list.
-       *  @param  l  An initializer_list.
-       *  @param  comp  A comparison functor.
-       *  @param  a  An allocator object.
+       *  @param  __l  An initializer_list.
+       *  @param  __comp  A comparison functor.
+       *  @param  __a  An allocator object.
        *
        *  Create a %multiset consisting of copies of the elements from
        *  the list.  This is linear in N if the list is already sorted,
-       *  and NlogN otherwise (where N is @a l.size()).
+       *  and NlogN otherwise (where N is @a __l.size()).
        */
       multiset(initializer_list<value_type> __l,
 	       const _Compare& __comp = _Compare(),
 	       const allocator_type& __a = allocator_type())
-      : _M_t(__comp, __a)
+      : _M_t(__comp, _Key_alloc_type(__a))
       { _M_t._M_insert_equal(__l.begin(), __l.end()); }
 #endif
 
       /**
        *  @brief  %Multiset assignment operator.
-       *  @param  x  A %multiset of identical element and allocator types.
+       *  @param  __x  A %multiset of identical element and allocator types.
        *
-       *  All the elements of @a x are copied, but unlike the copy constructor,
-       *  the allocator object is not copied.
+       *  All the elements of @a __x are copied, but unlike the copy
+       *  constructor, the allocator object is not copied.
        */
       multiset&
       operator=(const multiset& __x)
@@ -223,13 +232,14 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
 	return *this;
       }
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
+#if __cplusplus >= 201103L
       /**
        *  @brief  %Multiset move assignment operator.
-       *  @param  x  A %multiset of identical element and allocator types.
+       *  @param  __x  A %multiset of identical element and allocator types.
        *
-       *  The contents of @a x are moved into this %multiset (without copying).
-       *  @a x is a valid, but unspecified %multiset.
+       *  The contents of @a __x are moved into this %multiset
+       *  (without copying).  @a __x is a valid, but unspecified
+       *  %multiset.
        */
       multiset&
       operator=(multiset&& __x)
@@ -243,10 +253,10 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
 
       /**
        *  @brief  %Multiset list assignment operator.
-       *  @param  l  An initializer_list.
+       *  @param  __l  An initializer_list.
        *
        *  This function fills a %multiset with copies of the elements in the
-       *  initializer list @a l.
+       *  initializer list @a __l.
        *
        *  Note that the assignment completely changes the %multiset and
        *  that the resulting %multiset's size is the same as the number
@@ -273,8 +283,8 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       { return _M_t.key_comp(); }
       ///  Returns the memory allocation object.
       allocator_type
-      get_allocator() const
-      { return _M_t.get_allocator(); }
+      get_allocator() const _GLIBCXX_NOEXCEPT
+      { return allocator_type(_M_t.get_allocator()); }
 
       /**
        *  Returns a read-only (constant) iterator that points to the first
@@ -282,7 +292,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  according to the keys.
        */
       iterator
-      begin() const
+      begin() const _GLIBCXX_NOEXCEPT
       { return _M_t.begin(); }
 
       /**
@@ -291,7 +301,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  according to the keys.
        */
       iterator
-      end() const
+      end() const _GLIBCXX_NOEXCEPT
       { return _M_t.end(); }
 
       /**
@@ -300,7 +310,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  according to the keys.
        */
       reverse_iterator
-      rbegin() const
+      rbegin() const _GLIBCXX_NOEXCEPT
       { return _M_t.rbegin(); }
 
       /**
@@ -309,17 +319,17 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  according to the keys.
        */
       reverse_iterator
-      rend() const
+      rend() const _GLIBCXX_NOEXCEPT
       { return _M_t.rend(); }
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
+#if __cplusplus >= 201103L
       /**
        *  Returns a read-only (constant) iterator that points to the first
        *  element in the %multiset.  Iteration is done in ascending order
        *  according to the keys.
        */
       iterator
-      cbegin() const
+      cbegin() const noexcept
       { return _M_t.begin(); }
 
       /**
@@ -328,7 +338,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  according to the keys.
        */
       iterator
-      cend() const
+      cend() const noexcept
       { return _M_t.end(); }
 
       /**
@@ -337,7 +347,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  according to the keys.
        */
       reverse_iterator
-      crbegin() const
+      crbegin() const noexcept
       { return _M_t.rbegin(); }
 
       /**
@@ -346,28 +356,28 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  according to the keys.
        */
       reverse_iterator
-      crend() const
+      crend() const noexcept
       { return _M_t.rend(); }
 #endif
 
       ///  Returns true if the %set is empty.
       bool
-      empty() const
+      empty() const _GLIBCXX_NOEXCEPT
       { return _M_t.empty(); }
 
       ///  Returns the size of the %set.
       size_type
-      size() const
+      size() const _GLIBCXX_NOEXCEPT
       { return _M_t.size(); }
 
       ///  Returns the maximum size of the %set.
       size_type
-      max_size() const
+      max_size() const _GLIBCXX_NOEXCEPT
       { return _M_t.max_size(); }
 
       /**
        *  @brief  Swaps data with another %multiset.
-       *  @param  x  A %multiset of the same element and allocator types.
+       *  @param  __x  A %multiset of the same element and allocator types.
        *
        *  This exchanges the elements between two multisets in constant time.
        *  (It is only swapping a pointer, an integer, and an instance of the @c
@@ -381,9 +391,57 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       { _M_t.swap(__x._M_t); }
 
       // insert/erase
+#if __cplusplus >= 201103L
+      /**
+       *  @brief Builds and inserts an element into the %multiset.
+       *  @param  __args  Arguments used to generate the element instance to be
+       *                 inserted.
+       *  @return An iterator that points to the inserted element.
+       *
+       *  This function inserts an element into the %multiset.  Contrary
+       *  to a std::set the %multiset does not rely on unique keys and thus
+       *  multiple copies of the same element can be inserted.
+       *
+       *  Insertion requires logarithmic time.
+       */
+      template<typename... _Args>
+	iterator
+	emplace(_Args&&... __args)
+	{ return _M_t._M_emplace_equal(std::forward<_Args>(__args)...); }
+
+      /**
+       *  @brief Builds and inserts an element into the %multiset.
+       *  @param  __pos  An iterator that serves as a hint as to where the
+       *                element should be inserted.
+       *  @param  __args  Arguments used to generate the element instance to be
+       *                 inserted.
+       *  @return An iterator that points to the inserted element.
+       *
+       *  This function inserts an element into the %multiset.  Contrary
+       *  to a std::set the %multiset does not rely on unique keys and thus
+       *  multiple copies of the same element can be inserted.
+       *
+       *  Note that the first parameter is only a hint and can potentially
+       *  improve the performance of the insertion process.  A bad hint would
+       *  cause no gains in efficiency.
+       *
+       *  See http://gcc.gnu.org/onlinedocs/libstdc++/manual/bk01pt07ch17.html
+       *  for more on @a hinting.
+       *
+       *  Insertion requires logarithmic time (if the hint is not taken).
+       */
+      template<typename... _Args>
+	iterator
+	emplace_hint(const_iterator __pos, _Args&&... __args)
+	{
+	  return _M_t._M_emplace_hint_equal(__pos,
+					    std::forward<_Args>(__args)...);
+	}
+#endif
+
       /**
        *  @brief Inserts an element into the %multiset.
-       *  @param  x  Element to be inserted.
+       *  @param  __x  Element to be inserted.
        *  @return An iterator that points to the inserted element.
        *
        *  This function inserts an element into the %multiset.  Contrary
@@ -396,11 +454,17 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       insert(const value_type& __x)
       { return _M_t._M_insert_equal(__x); }
 
+#if __cplusplus >= 201103L
+      iterator
+      insert(value_type&& __x)
+      { return _M_t._M_insert_equal(std::move(__x)); }
+#endif
+
       /**
        *  @brief Inserts an element into the %multiset.
-       *  @param  position  An iterator that serves as a hint as to where the
+       *  @param  __position  An iterator that serves as a hint as to where the
        *                    element should be inserted.
-       *  @param  x  Element to be inserted.
+       *  @param  __x  Element to be inserted.
        *  @return An iterator that points to the inserted element.
        *
        *  This function inserts an element into the %multiset.  Contrary
@@ -417,14 +481,20 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  Insertion requires logarithmic time (if the hint is not taken).
        */
       iterator
-      insert(iterator __position, const value_type& __x)
+      insert(const_iterator __position, const value_type& __x)
       { return _M_t._M_insert_equal_(__position, __x); }
 
+#if __cplusplus >= 201103L
+      iterator
+      insert(const_iterator __position, value_type&& __x)
+      { return _M_t._M_insert_equal_(__position, std::move(__x)); }
+#endif
+
       /**
        *  @brief A template function that tries to insert a range of elements.
-       *  @param  first  Iterator pointing to the start of the range to be
-       *                 inserted.
-       *  @param  last  Iterator pointing to the end of the range.
+       *  @param  __first  Iterator pointing to the start of the range to be
+       *                   inserted.
+       *  @param  __last  Iterator pointing to the end of the range.
        *
        *  Complexity similar to that of the range constructor.
        */
@@ -433,11 +503,11 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
         insert(_InputIterator __first, _InputIterator __last)
         { _M_t._M_insert_equal(__first, __last); }
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
+#if __cplusplus >= 201103L
       /**
        *  @brief Attempts to insert a list of elements into the %multiset.
-       *  @param  list  A std::initializer_list<value_type> of elements
-       *                to be inserted.
+       *  @param  __l  A std::initializer_list<value_type> of elements
+       *               to be inserted.
        *
        *  Complexity similar to that of the range constructor.
        */
@@ -446,12 +516,12 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       { this->insert(__l.begin(), __l.end()); }
 #endif
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
+#if __cplusplus >= 201103L
       // _GLIBCXX_RESOLVE_LIB_DEFECTS
       // DR 130. Associative erase should return an iterator.
       /**
        *  @brief Erases an element from a %multiset.
-       *  @param  position  An iterator pointing to the element to be erased.
+       *  @param  __position  An iterator pointing to the element to be erased.
        *  @return An iterator pointing to the element immediately following
        *          @a position prior to the element being erased. If no such 
        *          element exists, end() is returned.
@@ -463,12 +533,12 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  responsibility.
        */
       iterator
-      erase(iterator __position)
+      erase(const_iterator __position)
       { return _M_t.erase(__position); }
 #else
       /**
        *  @brief Erases an element from a %multiset.
-       *  @param  position  An iterator pointing to the element to be erased.
+       *  @param  __position  An iterator pointing to the element to be erased.
        *
        *  This function erases an element, pointed to by the given iterator,
        *  from a %multiset.  Note that this function only erases the element,
@@ -483,7 +553,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
 
       /**
        *  @brief Erases elements according to the provided key.
-       *  @param  x  Key of element to be erased.
+       *  @param  __x  Key of element to be erased.
        *  @return  The number of elements erased.
        *
        *  This function erases all elements located by the given key from a
@@ -496,23 +566,25 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       erase(const key_type& __x)
       { return _M_t.erase(__x); }
 
-#ifdef __GXX_EXPERIMENTAL_CXX0X__
+#if __cplusplus >= 201103L
       // _GLIBCXX_RESOLVE_LIB_DEFECTS
       // DR 130. Associative erase should return an iterator.
       /**
        *  @brief Erases a [first,last) range of elements from a %multiset.
-       *  @param  first  Iterator pointing to the start of the range to be
-       *                 erased.
-       *  @param  last  Iterator pointing to the end of the range to be erased.
+       *  @param  __first  Iterator pointing to the start of the range to be
+       *                   erased.
+       *  @param __last Iterator pointing to the end of the range to
+       *                be erased.
        *  @return The iterator @a last.
        *
        *  This function erases a sequence of elements from a %multiset.
        *  Note that this function only erases the elements, and that if
        *  the elements themselves are pointers, the pointed-to memory is not
-       *  touched in any way.  Managing the pointer is the user's responsibility.
+       *  touched in any way.  Managing the pointer is the user's
+       *  responsibility.
        */
       iterator
-      erase(iterator __first, iterator __last)
+      erase(const_iterator __first, const_iterator __last)
       { return _M_t.erase(__first, __last); }
 #else
       /**
@@ -524,7 +596,8 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  This function erases a sequence of elements from a %multiset.
        *  Note that this function only erases the elements, and that if
        *  the elements themselves are pointers, the pointed-to memory is not
-       *  touched in any way.  Managing the pointer is the user's responsibility.
+       *  touched in any way.  Managing the pointer is the user's
+       *  responsibility.
        */
       void
       erase(iterator __first, iterator __last)
@@ -538,14 +611,14 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
        *  is the user's responsibility.
        */
       void
-      clear()
+      clear() _GLIBCXX_NOEXCEPT
       { _M_t.clear(); }
 
       // multiset operations:
 
       /**
        *  @brief Finds the number of elements with given key.
-       *  @param  x  Key of elements to be located.
+       *  @param  __x  Key of elements to be located.
        *  @return Number of elements with specified key.
        */
       size_type
@@ -557,7 +630,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       //@{
       /**
        *  @brief Tries to locate an element in a %set.
-       *  @param  x  Element to be located.
+       *  @param  __x  Element to be located.
        *  @return  Iterator pointing to sought-after element, or end() if not
        *           found.
        *
@@ -578,7 +651,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       //@{
       /**
        *  @brief Finds the beginning of a subsequence matching given key.
-       *  @param  x  Key to be located.
+       *  @param  __x  Key to be located.
        *  @return  Iterator pointing to first element equal to or greater
        *           than key, or end().
        *
@@ -599,7 +672,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       //@{
       /**
        *  @brief Finds the end of a subsequence matching given key.
-       *  @param  x  Key to be located.
+       *  @param  __x  Key to be located.
        *  @return Iterator pointing to the first element
        *          greater than key, or end().
        */
@@ -615,7 +688,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       //@{
       /**
        *  @brief Finds a subsequence matching given key.
-       *  @param  x  Key to be located.
+       *  @param  __x  Key to be located.
        *  @return  Pair of iterators that possibly points to the subsequence
        *           matching given key.
        *
@@ -635,6 +708,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
       std::pair<const_iterator, const_iterator>
       equal_range(const key_type& __x) const
       { return _M_t.equal_range(__x); }
+      //@}
 
       template<typename _K1, typename _C1, typename _A1>
         friend bool
@@ -649,8 +723,8 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
 
   /**
    *  @brief  Multiset equality comparison.
-   *  @param  x  A %multiset.
-   *  @param  y  A %multiset of the same type as @a x.
+   *  @param  __x  A %multiset.
+   *  @param  __y  A %multiset of the same type as @a __x.
    *  @return  True iff the size and elements of the multisets are equal.
    *
    *  This is an equivalence relation.  It is linear in the size of the
@@ -666,9 +740,9 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
 
   /**
    *  @brief  Multiset ordering relation.
-   *  @param  x  A %multiset.
-   *  @param  y  A %multiset of the same type as @a x.
-   *  @return  True iff @a x is lexicographically less than @a y.
+   *  @param  __x  A %multiset.
+   *  @param  __y  A %multiset of the same type as @a __x.
+   *  @return  True iff @a __x is lexicographically less than @a __y.
    *
    *  This is a total ordering relation.  It is linear in the size of the
    *  maps.  The elements must be comparable with @c <.
@@ -716,6 +790,7 @@ _GLIBCXX_BEGIN_NESTED_NAMESPACE(std, _GLIBCXX_STD_D)
 	 multiset<_Key, _Compare, _Alloc>& __y)
     { __x.swap(__y); }
 
-_GLIBCXX_END_NESTED_NAMESPACE
+_GLIBCXX_END_NAMESPACE_CONTAINER
+} // namespace std
 
 #endif /* _STL_MULTISET_H */
