@@ -1,6 +1,5 @@
 // -*- C++ -*- Manage the thread-local exception globals.
-// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2009
-// Free Software Foundation, Inc.
+// Copyright (C) 2001-2013 Free Software Foundation, Inc.
 //
 // This file is part of GCC.
 //
@@ -47,7 +46,7 @@ using namespace __cxxabiv1;
 namespace
 {
   abi::__cxa_eh_globals*
-  get_global() throw()
+  get_global() _GLIBCXX_NOTHROW
   {
     static __thread abi::__cxa_eh_globals global;
     return &global;
@@ -55,11 +54,11 @@ namespace
 } // anonymous namespace
 
 extern "C" __cxa_eh_globals*
-__cxxabiv1::__cxa_get_globals_fast() throw()
+__cxxabiv1::__cxa_get_globals_fast() _GLIBCXX_NOTHROW
 { return get_global(); }
 
 extern "C" __cxa_eh_globals*
-__cxxabiv1::__cxa_get_globals() throw()
+__cxxabiv1::__cxa_get_globals() _GLIBCXX_NOTHROW
 { return get_global(); }
 
 
@@ -110,7 +109,7 @@ struct __eh_globals_init
 static __eh_globals_init init;
 
 extern "C" __cxa_eh_globals*
-__cxxabiv1::__cxa_get_globals_fast() throw()
+__cxxabiv1::__cxa_get_globals_fast() _GLIBCXX_NOTHROW
 {
   __cxa_eh_globals* g;
   if (init._M_init)
@@ -121,7 +120,7 @@ __cxxabiv1::__cxa_get_globals_fast() throw()
 }
 
 extern "C" __cxa_eh_globals*
-__cxxabiv1::__cxa_get_globals() throw()
+__cxxabiv1::__cxa_get_globals() _GLIBCXX_NOTHROW
 {
   __cxa_eh_globals* g;
   if (init._M_init)
@@ -148,11 +147,11 @@ __cxxabiv1::__cxa_get_globals() throw()
 #else
 
 extern "C" __cxa_eh_globals*
-__cxxabiv1::__cxa_get_globals_fast() throw()
+__cxxabiv1::__cxa_get_globals_fast() _GLIBCXX_NOTHROW
 { return &eh_globals; }
 
 extern "C" __cxa_eh_globals*
-__cxxabiv1::__cxa_get_globals() throw()
+__cxxabiv1::__cxa_get_globals() _GLIBCXX_NOTHROW
 { return &eh_globals; }
 
 #endif
