@@ -1,6 +1,6 @@
 // 2003-02-26 Benjamin Kosnik <bkoz@redhat.com>
 
-// Copyright (C) 2003, 2009 Free Software Foundation, Inc.
+// Copyright (C) 2003-2013 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -25,8 +25,9 @@
 int main()
 {
   using namespace __gnu_test;
+
   // cplus-dem FAIL
-  verify_demangle("_Z1fA37_iPS_", "f(int [37], int (*) [37])");
+  verify_demangle("_Z1fPFPA1_ivE", "f(int (*(*)()) [1])");
 
   return 0;
 }
