@@ -1,11 +1,11 @@
 // 2003-03-01 gp dot bolton at computer dot org
 
-// Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
+// Copyright (C) 2003-2013 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,20 +14,11 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
 
-#include <testsuite_common_types.h>
+#include <testsuite_performance.h>
 
 template<typename Container, int Iter>
   void
@@ -51,11 +42,9 @@ main()
 #endif    
 
   typedef __gnu_test::maps<int, thread_type>::type container_types;
-
   typedef test_sequence<thread_type> test_type;
   test_type test("create");
-  __gnu_cxx::apply<test_type, container_types> applier;
-  applier(test);
+  __gnu_cxx::typelist::apply(test, container_types());
 
   return 0;
 }
